package com.nokia.domainb.capability.controller;

import com.alibaba.fastjson2.JSON;
import com.nokia.domainb.DomainBApplication;
import com.nokia.domainb.capability.dto.RouteDTO;
import com.nokia.domainb.capability.dto.SaDTO;
import com.nokia.domainb.common.http.R;
import com.nokia.domainb.util.Utils;
import de.siegmar.fastcsv.reader.CsvReader;
import de.siegmar.fastcsv.reader.CsvRow;
import de.siegmar.fastcsv.writer.CsvWriter;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.scheduling.concurrent.ThreadPoolTaskScheduler;
import org.springframework.web.client.RestTemplate;

import java.io.IOException;
import java.io.OutputStreamWriter;
import java.nio.charset.StandardCharsets;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.time.LocalDateTime;
import java.util.Set;

@Slf4j
@SpringBootTest(classes = DomainBApplication.class)
class DomainControllerTest {
    @Autowired
    private RestTemplate restTemplate;
    @Autowired
    ThreadPoolTaskScheduler threadPoolTaskScheduler;

    @Test
    void testRoute() {
        String url = "http://133.96.94.108:12091/domainB/api/route/";
        Set<String> phones = Utils.readPhones();
        for (String t : phones) {
            String[] a = t.split(",");
            String phone = a[0];
            threadPoolTaskScheduler.submit(new RouteTask(restTemplate, phone, url));
        }
        while (threadPoolTaskScheduler.getActiveCount() > 0) {
        }
        String resultPath = "z:/route/路由查询10个线程10000条数据测试结果.csv";
//        String resultPath = "z:/route/路由查询10个线程1000条数据测试结果.csv";
//        String resultPath = "z:/route/result.csv";
        OutputStreamWriter osw = null;
        try {
            osw = new OutputStreamWriter(Files.newOutputStream(Paths.get(resultPath)), "GBK");
        } catch (IOException e) {
            e.printStackTrace();
            return;
        }
        CsvWriter csvWriter = CsvWriter.builder().build(osw);
        csvWriter.writeRow("入参", "时间", "调用时长", "结果");
        for (String tt : phones) {
            String[] a = tt.split(",");
            String phone = a[0];
            String singlePath = "z:/route/single/" + phone + ".csv";
            Path path = Paths.get(singlePath);
            try (CsvReader csvReader = CsvReader.builder().build(path, StandardCharsets.UTF_8)) {
                for (CsvRow row : csvReader) {
                    csvWriter.writeRow(row.getField(0), row.getField(1), row.getField(2), row.getField(3));
                }
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        try {
            csvWriter.close();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    @Test
    void testSa() {
        String url = "http://133.96.94.108:12091/domainB/api/sa/";
        R r = restTemplate.postForObject(url, new SaDTO("test", "18503369666"), R.class);
        System.out.println(r);
    }
//
//    @Test
//    void testSaAndVolte() {
//        String url = "http://133.96.94.108:12091/domainB/api/product/saAndVolte/";
//        R r = restTemplate.postForObject(url, new SaAndVolteDTO("test", "18503369666"), R.class);
//        System.out.println(r);
//    }

    @AllArgsConstructor
    class RouteTask implements Runnable {
        private RestTemplate restTemplate;
        private String phoneNumber;
        private String url;

        @Override
        public void run() {
            RouteDTO dto = new RouteDTO("test", phoneNumber);
            LocalDateTime startTime = LocalDateTime.now();
            long startTimestamp = System.currentTimeMillis();
            R r = restTemplate.postForObject(url, dto, R.class);
            long endTimestamp = System.currentTimeMillis();
            long cost = endTimestamp - startTimestamp;
            String filePath = "z:/route/single/" + phoneNumber + ".csv";
            try (OutputStreamWriter osw = new OutputStreamWriter(Files.newOutputStream(Paths.get(filePath)),
                    StandardCharsets.UTF_8);
                 CsvWriter csv = CsvWriter.builder().build(osw);) {
                csv.writeRow(JSON.toJSONString(dto), startTime.toString(), String.valueOf(cost), JSON.toJSONString(r));
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
    }
}

